"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver_1 = __importDefault(require("semver"));
const variablePrefix = 'package';
const packageFile = '/package.json';
class Plugin {
    constructor(serverless) {
        this.serverless = serverless;
        this.packageJson = require(`${process.cwd()}/${packageFile}`);
        this.bindVariableResolution();
    }
    bindVariableResolution() {
        const delegate = this.serverless.variables.getValueFromSource.bind(this.serverless.variables);
        this.serverless.variables.getValueFromSource = (variable) => {
            if (variable.startsWith(`${variablePrefix}:`)) {
                return this.resolveVariable(variable);
            }
            return delegate(variable);
        };
    }
    resolveVariable(variable) {
        const key = variable.split(`${variablePrefix}:`)[1];
        this.serverless.cli.log(`Resolving ${key} from package.json`);
        if (key.startsWith('semver.'))
            return this.resolveSemverSection(key);
        return this.packageJson[key] || '';
    }
    resolveSemverSection(key) {
        const { version } = this.packageJson;
        const semverSection = key.split('semver.')[1];
        if (!version) {
            this.serverless.cli.log(`Could not get ${key} from package.json, package.json is missing key 'version'`);
            return '';
        }
        this.serverless.cli.log(`Resolving semver ${semverSection} from: ${version}`);
        if (typeof semver_1.default[semverSection] === 'function') {
            return semver_1.default[semverSection](version);
        }
        this.serverless.cli.log(`Could not get ${key}, ${semverSection} is not a function.`);
        return '';
    }
}
exports.default = Plugin;
module.exports = Plugin;
//# sourceMappingURL=index.js.map